%%
%% This is file `bxjsja-modern.def',
%% generated with the docstrip utility.
%%
%% The original source files were:
%%
%% bxjscls.dtx  (with options: `drv,modern')
%% 
%% IMPORTANT NOTICE:
%% 
%% For the copyright see the source file.
%% 
%% Any modified versions of this file must be renamed
%% with new filenames distinct from bxjsja-modern.def.
%% 
%% For distribution of the original source see the terms
%% for copying and modification in the file bxjscls.dtx.
%% 
%% This generated file may be distributed as long as the
%% original source files, as listed above, are part of the
%% same distribution. (The sources need not necessarily be
%% in the same archive or directory.)
\ProvidesFile{bxjsja-modern.def}
  [2024/07/19 v2.9c BXJS document classes]
\input{bxjsja-standard.def}
\ifnum0\if x\jsEngine1\fi\if l\jsEngine1\fi=\z@
\def\encodingdefault{T1}%
\input{t1enc.def}%
\fontencoding\encodingdefault\selectfont
\fi
\ifnum0\if x\jsEngine1\fi\if l\jsEngine1\fi=\z@
\renewcommand{\rmdefault}{lmr}
\renewcommand{\sfdefault}{lmss}
\renewcommand{\ttdefault}{lmtt}
\fi
\DeclareFontShape{OMX}{cmex}{m}{n}{%
  <-7.5>cmex7<7.5-8.5>cmex8%
  <8.5-9.5>cmex9<9.5->cmex10}{}%
\expandafter\let\csname OMX/cmex/m/n/10\endcsname\relax
\def\cmex@opt{10}
\ifx\@IncludeInRelease\@undefined
\RequirePackage{fixltx2e}
\fi
\RequirePackage{bxjscjkcat}
\endinput
%%
%% End of file `bxjsja-modern.def'.
