%%
%% This is file `bxjsja-minimal.def',
%% generated with the docstrip utility.
%%
%% The original source files were:
%%
%% bxjscls.dtx  (with options: `drv,minimal')
%% 
%% IMPORTANT NOTICE:
%% 
%% For the copyright see the source file.
%% 
%% Any modified versions of this file must be renamed
%% with new filenames distinct from bxjsja-minimal.def.
%% 
%% For distribution of the original source see the terms
%% for copying and modification in the file bxjscls.dtx.
%% 
%% This generated file may be distributed as long as the
%% original source files, as listed above, are part of the
%% same distribution. (The sources need not necessarily be
%% in the same archive or directory.)
\ProvidesFile{bxjsja-minimal.def}
  [2024/07/19 v2.9c BXJS document classes]
%% このファイルは日本語文字を含みます
\let\bxjs@tmpa\relax
\ifx J\jsEngine \def\bxjs@tmpa{NTT-jTeX}\fi
\ifx O\jsEngine \def\bxjs@tmpa{Omega}\fi
\ifx\bxjs@tmpa\relax \expandafter\@gobble
\else
  \ClassError\bxjs@clsname
   {The engine in use (\bxjs@tmpa) is not supported}
   {It's a fatal error. I'll quit right now.}
  \expandafter\@firstofone
\fi{\endinput\@@end}
\def\DeclareJaTextFontCommand#1#2{%
  \DeclareRobustCommand#1[1]{%
    \relax
    \ifmmode \expandafter\nfss@text \fi
    {#2##1}}%
}
\def\DeclareJaMathFontCommand#1#2{%
  \DeclareRobustCommand#1[1]{%
    \relax
    \ifmmode\else \non@alpherr{#1\space}\fi
    \nfss@text{\fontfamily\familydefault
        \fontseries{m}\fontshape{n}\selectfont\relax
        #2##1}%
  }%
}
\long\def\bxjs@@CSsfdefault{\sfdefault}%
\@onlypreamble\bxjs@if@sf@default
\def\bxjs@if@sf@default#1{%
  \ifx\familydefault\bxjs@@CSsfdefault#1\fi
  \g@addto@macro\bxjs@begin@document@hook{%
    \ifx\familydefault\bxjs@@CSsfdefault#1\fi}%
}
\@tempdima\p@ \bxjs@invscale\@tempdima\jsScale
\edef\jsInverseScale{\strip@pt\@tempdima}
\def\jsLetHeadChar#1#2{%
  \begingroup
    \escapechar=`\\ %
    \let\bxjs@tmpa={% brace-match-hack
    \bxjs@let@hchar@exp#2}%
  \endgroup
  \let#1\bxjs@g@tmpa}
\def\bxjs@let@hchar@exp{%
  \futurelet\@let@token\bxjs@let@hchar@exp@a}
\def\bxjs@let@hchar@exp@a{%
  \bxjs@cond\ifcat\noexpand\@let@token\bgroup\fi{% 波括弧
    \bxjs@let@hchar@out\let\relax
  }{\bxjs@cond\ifcat\noexpand\@let@token\@sptoken\fi{% 空白
    \bxjs@let@hchar@out\let\space%
  }{\bxjs@cond\if\noexpand\@let@token\@backslashchar\fi{% バックスラッシュ
    \bxjs@let@hchar@out\let\@backslashchar
  }{\bxjs@let@hchar@exp@b}}}}
\def\bxjs@let@hchar@exp@b#1{%
  \expandafter\bxjs@let@hchar@exp@c\string#1?\@nil#1}
\def\bxjs@let@hchar@exp@c#1#2\@nil{%
  \bxjs@cond\if#1\@backslashchar\fi{% 制御綴
    \bxjs@cond\expandafter\ifx\noexpand\@let@token\@let@token\fi{%
      \bxjs@let@hchar@out\let\relax
    }{%else
      \expandafter\bxjs@let@hchar@exp
    }%
  }{%else
    \bxjs@let@hchar@chr#1%
  }}
\def\bxjs@let@hchar@chr#1{%
  \bxjs@let@hchar@out\def{{#1}}}
\def\bxjs@let@hchar@out#1#2{%
  \global#1\bxjs@g@tmpa#2\relax
  \toks@\bgroup}% skip to right brace
\chardef\bxjs@let@hchar@csta=128
\chardef\bxjs@let@hchar@cstb=192
\chardef\bxjs@let@hchar@cstc=224
\chardef\bxjs@let@hchar@cstd=240
\chardef\bxjs@let@hchar@cste=248
\let\bxjs@let@hchar@chr@ue@a\bxjs@let@hchar@chr
\def\bxjs@let@hchar@chr@ue#1{%
  \@tempcnta=`#1\relax
  \bxjs@cond\ifnum\@tempcnta<\bxjs@let@hchar@csta\fi{%
    \bxjs@let@hchar@chr@ue@a#1%
  }{\bxjs@cond\ifnum\@tempcnta<\bxjs@let@hchar@cstb\fi{%
    \bxjs@let@hchar@out\let\relax
  }{\bxjs@cond\ifnum\@tempcnta<\bxjs@let@hchar@cstc\fi{%
    \bxjs@let@hchar@chr@ue@b
  }{\bxjs@cond\ifnum\@tempcnta<\bxjs@let@hchar@cstd\fi{%
    \bxjs@let@hchar@chr@ue@c
  }{\bxjs@cond\ifnum\@tempcnta<\bxjs@let@hchar@cste\fi{%
    \bxjs@let@hchar@chr@ue@d
  }{%else
    \bxjs@let@hchar@out\let\relax
  }}}}}}
\def\bxjs@let@hchar@chr@ue@a#1{%
  \bxjs@let@hchar@out\def{{#1}}}
\def\bxjs@let@hchar@chr@ue@b#1#2{%
  \bxjs@let@hchar@out\def{{#1#2}}}
\def\bxjs@let@hchar@chr@ue@c#1#2#3{%
  \bxjs@let@hchar@out\def{{#1#2#3}}}
\def\bxjs@let@hchar@chr@ue@d#1#2#3#4{%
  \bxjs@let@hchar@out\def{{#1#2#3#4}}}
\ifx j\jsEngine
\def\bxjs@let@hchar@chr@pp#1#2{%
  \expandafter\bxjs@let@hchar@chr@pp@a\meaning#2\relax#1#2}
\def\bxjs@let@hchar@chr@pp@a#1#2\relax#3#4{%
  \bxjs@cond\if#1k\fi{%
    \bxjs@let@hchar@out\def{{#4}}%
  }{%else
    \bxjs@let@hchar@chr@ue#3#4%
  }}
\let\bxjs@let@hchar@chr\bxjs@let@hchar@chr@pp
\edef\jsc@JYn{\ifjsWithupTeX JY2\else JY1\fi}
\edef\jsc@JTn{\ifjsWithupTeX JT2\else JT1\fi}
\edef\jsc@pfx@{\ifjsWithupTeX u\fi}
\@onlypreamble\bxjs@declarefontshape
\ifjsWithupTeX
\def\bxjs@declarefontshape{%
\DeclareFontShape{JY2}{mc}{m}{n}{<->s*[\bxjs@scale]upjpnrm-h}{}%
\DeclareFontShape{JY2}{gt}{m}{n}{<->s*[\bxjs@scale]upjpngt-h}{}%
\DeclareFontShape{JT2}{mc}{m}{n}{<->s*[\bxjs@scale]upjpnrm-v}{}%
\DeclareFontShape{JT2}{gt}{m}{n}{<->s*[\bxjs@scale]upjpngt-v}{}%
}
\def\bxjs@sizereference{upjisr-h}
\else
\def\bxjs@declarefontshape{%
\DeclareFontShape{JY1}{mc}{m}{n}{<->s*[\bxjs@scale]jis}{}%
\DeclareFontShape{JY1}{gt}{m}{n}{<->s*[\bxjs@scale]jisg}{}%
\DeclareFontShape{JT1}{mc}{m}{n}{<->s*[\bxjs@scale]tmin10}{}%
\DeclareFontShape{JT1}{gt}{m}{n}{<->s*[\bxjs@scale]tgoth10}{}%
}
\def\bxjs@sizereference{jis}
\fi
\def\bxjs@next#1/#2/#3/#4/#5\relax{%
  \def\bxjs@tmpb{#5}}
\ifjsWithpTeXng \def\bxjs@tmpb{10}%
\else
\expandafter\expandafter\expandafter\bxjs@next
 \expandafter\string\the\jfont\relax
\fi
\@for\bxjs@tmpa:={\jsc@JYn/mc/m/n,\jsc@JYn/gt/m/n,%
               \jsc@JTn/mc/m/n,\jsc@JTn/gt/m/n}\do
  {\expandafter\let\csname\bxjs@tmpa/10\endcsname=\@undefined
   \expandafter\let\csname\bxjs@tmpa/\bxjs@tmpb\endcsname=\@undefined}
\begingroup
  \font\bxjs@tmpa=\bxjs@sizereference\space at 10pt
  \setbox\z@\hbox{\bxjs@tmpa\char\jis"2121\relax}
  \ifdim\wd\z@=10pt
    \global\let\bxjs@scale\jsScale
  \else
    \edef\bxjs@tmpa{\strip@pt\wd\z@}
    \@tempdima=10pt \@tempdima=\jsScale\@tempdima
    \bxjs@invscale\@tempdima\bxjs@tmpa
    \xdef\bxjs@scale{\strip@pt\@tempdima}
  \fi
\endgroup
\bxjs@declarefontshape
\DeclareFontShape{\jsc@JYn}{mc}{m}{it}{<->ssub*mc/m/n}{}
\DeclareFontShape{\jsc@JYn}{mc}{m}{sl}{<->ssub*mc/m/n}{}
\DeclareFontShape{\jsc@JYn}{mc}{m}{sc}{<->ssub*mc/m/n}{}
\DeclareFontShape{\jsc@JYn}{gt}{m}{it}{<->ssub*gt/m/n}{}
\DeclareFontShape{\jsc@JYn}{gt}{m}{sl}{<->ssub*gt/m/n}{}
\DeclareFontShape{\jsc@JYn}{mc}{bx}{it}{<->ssub*gt/m/n}{}
\DeclareFontShape{\jsc@JYn}{mc}{bx}{sl}{<->ssub*gt/m/n}{}
\DeclareFontShape{\jsc@JYn}{gt}{bx}{it}{<->ssub*gt/m/n}{}
\DeclareFontShape{\jsc@JYn}{gt}{bx}{sl}{<->ssub*gt/m/n}{}
\DeclareFontShape{\jsc@JYn}{mc}{b}{n}{<->ssub*mc/bx/n}{}
\DeclareFontShape{\jsc@JYn}{mc}{b}{it}{<->ssub*mc/bx/n}{}
\DeclareFontShape{\jsc@JYn}{mc}{b}{sl}{<->ssub*mc/bx/n}{}
\DeclareFontShape{\jsc@JYn}{gt}{b}{n}{<->ssub*gt/bx/n}{}
\DeclareFontShape{\jsc@JYn}{gt}{b}{it}{<->ssub*gt/bx/n}{}
\DeclareFontShape{\jsc@JYn}{gt}{b}{sl}{<->ssub*gt/bx/n}{}
\DeclareFontShape{\jsc@JTn}{mc}{m}{it}{<->ssub*mc/m/n}{}
\DeclareFontShape{\jsc@JTn}{mc}{m}{sl}{<->ssub*mc/m/n}{}
\DeclareFontShape{\jsc@JTn}{mc}{m}{sc}{<->ssub*mc/m/n}{}
\DeclareFontShape{\jsc@JTn}{gt}{m}{it}{<->ssub*gt/m/n}{}
\DeclareFontShape{\jsc@JTn}{gt}{m}{sl}{<->ssub*gt/m/n}{}
\DeclareFontShape{\jsc@JTn}{mc}{bx}{it}{<->ssub*gt/m/n}{}
\DeclareFontShape{\jsc@JTn}{mc}{bx}{sl}{<->ssub*gt/m/n}{}
\DeclareFontShape{\jsc@JTn}{gt}{bx}{it}{<->ssub*gt/m/n}{}
\DeclareFontShape{\jsc@JTn}{gt}{bx}{sl}{<->ssub*gt/m/n}{}
\DeclareFontShape{\jsc@JTn}{mc}{b}{n}{<->ssub*mc/bx/n}{}
\DeclareFontShape{\jsc@JTn}{mc}{b}{it}{<->ssub*mc/bx/n}{}
\DeclareFontShape{\jsc@JTn}{mc}{b}{sl}{<->ssub*mc/bx/n}{}
\DeclareFontShape{\jsc@JTn}{gt}{b}{n}{<->ssub*gt/bx/n}{}
\DeclareFontShape{\jsc@JTn}{gt}{b}{it}{<->ssub*gt/bx/n}{}
\DeclareFontShape{\jsc@JTn}{gt}{b}{sl}{<->ssub*gt/bx/n}{}
\@ifl@t@r\fmtversion{2020/10/01}
    {\jsc@needsp@tchfalse}{\jsc@needsp@tchtrue}
\ifjsc@needsp@tch          % --- for 2020-02-02 or older BEGIN
\ifx\@rmfamilyhook\@undefined  % old
\DeclareRobustCommand\rmfamily
  {\not@math@alphabet\rmfamily\mathrm
   \romanfamily\rmdefault\kanjifamily\mcdefault\selectfont}
\DeclareRobustCommand\sffamily
  {\not@math@alphabet\sffamily\mathsf
   \romanfamily\sfdefault\kanjifamily\gtdefault\selectfont}
\DeclareRobustCommand\ttfamily
  {\not@math@alphabet\ttfamily\mathtt
   \romanfamily\ttdefault\kanjifamily\gtdefault\selectfont}
\g@addto@macro\bxjs@begin@document@hook{%
  \ifx\mweights@init\@undefined\else  % mweights.sty is loaded
    % my definitions above should have been overwritten, recover it!
    % \selectfont is executed twice but I don't care about speed...
    \expandafter\g@addto@macro\csname rmfamily \endcsname
      {\kanjifamily\mcdefault\selectfont}%
    \expandafter\g@addto@macro\csname sffamily \endcsname
      {\kanjifamily\gtdefault\selectfont}%
    \expandafter\g@addto@macro\csname ttfamily \endcsname
      {\kanjifamily\gtdefault\selectfont}%
  \fi}
\else                          % 2020-02-02
\g@addto@macro\@rmfamilyhook
  {\prepare@family@series@update@kanji{mc}\mcdefault}
\g@addto@macro\@sffamilyhook
  {\prepare@family@series@update@kanji{gt}\gtdefault}
\g@addto@macro\@ttfamilyhook
  {\prepare@family@series@update@kanji{gt}\gtdefault}
\fi
\else % --- for 2020-02-02 or older END & for 2020-10-01 BEGIN
\AddToHook{rmfamily}%
  {\prepare@family@series@update@kanji{mc}\mcdefault}
\AddToHook{sffamily}%
  {\prepare@family@series@update@kanji{gt}\gtdefault}
\AddToHook{ttfamily}%
  {\prepare@family@series@update@kanji{gt}\gtdefault}
\fi   % --- for 2020-10-01 END
\ifx\DeclareFixJFMCJKTextFontCommand\@undefined
\DeclareJaTextFontCommand{\textmc}{\mcfamily}
\DeclareJaTextFontCommand{\textgt}{\gtfamily}
\fi
\bxjs@if@sf@default{%
  \renewcommand\kanjifamilydefault{\gtdefault}}
\selectfont
\def\bxjs@parse@qh#1{\let\bxjs@tmpb\relax}
\let\bxjs@parse@qh@a\@undefined
\let\bxjs@parse@qh@b\@undefined
\prebreakpenalty\jis"2147=10000
\postbreakpenalty\jis"2148=10000
\prebreakpenalty\jis"2149=10000
\inhibitxspcode`！=1
\inhibitxspcode`〒=2
\xspcode`+=3
\xspcode`\%=3
\@tempcnta="80 \@whilenum\@tempcnta<"100 \do{%
  \xspcode\@tempcnta=3\advance\@tempcnta\@ne}
\let\jsInhibitGlueAtParTop\@inhibitglue
\begingroup
\catcode`\!=0
\gdef\bxjs@ptex@dir{%
  !iftdir t%
  !else!ifydir y%
  !else ?%
  !fi!fi}
\long\def\bxjs@tmpa{\hbox{%
  !ifydir \@textsuperscript{\normalfont\@thefnmark}%
  !else\hbox{\yoko\@textsuperscript{\normalfont\@thefnmark}}!fi}}
\ifx\@makefnmark\bxjs@tmpa
\long\gdef\@makefnmark{%
  !ifydir \hbox{}\hbox{\@textsuperscript{\normalfont\@thefnmark}}\hbox{}%
  !else\hbox{\yoko\@textsuperscript{\normalfont\@thefnmark}}!fi}
\fi
\endgroup
\@namedef{ver@minijs.sty}{}
\else\if \if p\jsEngine T\else\if n\jsEngine T\else F\fi\fi T
\let\bxjs@let@hchar@chr\bxjs@let@hchar@chr@ue
\@onlypreamble\bxjs@cjk@loaded
\def\bxjs@cjk@loaded{%
  \def\@footnotemark{%
    \leavevmode
    \ifhmode
      \edef\@x@sf{\the\spacefactor}%
      \ifdim\lastkern>\z@\ifdim\lastkern<5sp\relax
         \unkern\unkern
         \ifdim\lastskip>\z@ \unskip \fi
      \fi\fi
      \nobreak
    \fi
    \@makefnmark
    \ifhmode \spacefactor\@x@sf \fi
    \relax}%
  \let\bxjs@cjk@loaded\relax
}
\g@addto@macro\bxjs@begin@document@hook{%
  \@ifpackageloaded{CJK}{%
    \bxjs@cjk@loaded
  }{}%
}
\else\ifx x\jsEngine
\def\bxjs@let@hchar@chr#1{%
  \@tempcnta`#1\relax \divide\@tempcnta"800\relax
  \bxjs@cond\ifnum\@tempcnta=27 \fi{%
    \bxjs@let@hchar@chr@xe
  }{\bxjs@let@hchar@out\def{{#1}}}}
\def\bxjs@let@hchar@chr@xe#1{%
  \lccode`0=`#1\relax
  \lowercase{\bxjs@let@hchar@out\def{{0}}}}
\@onlypreamble\bxjs@do@precisetext
\ifx\XeTeXgenerateactualtext\@undefined\else
  \def\bxjs@do@precisetext{%
    \XeTeXgenerateactualtext=\@ne}
\fi
\@onlypreamble\bxjs@do@simplejasetup
\def\bxjs@do@simplejasetup{%
  \@namedef{bxjs@zeroglue/0.0pt}{T}%
  \ifnum\XeTeXinterchartokenstate>\z@
  \else\expandafter\ifx\csname bxjs@zeroglue/\the\XeTeXlinebreakskip\endcsname\relax\else
    \jsSimpleJaSetup
    \ClassInfo\bxjs@clsname
     {'\string\jsSimpleJaSetup' is applied\@gobble}%
  \fi\fi}
\newcommand*{\jsSimpleJaSetup}{%
  \XeTeXlinebreaklocale "ja"\relax
  \XeTeXlinebreakskip=0pt plus 1pt minus 0.1pt\relax
  \XeTeXlinebreakpenalty=0\relax}
\fi\fi\fi
\ifx\bxjs@do@simplejasetup\@undefined\else
  \g@addto@macro\bxjs@begin@document@hook{%
    \ifbxjs@simplejasetup
      \bxjs@do@simplejasetup
    \fi}
\fi
\ifbxjs@precisetext
  \ifx\bxjs@do@precisetext\@undefined
    \ClassWarning\bxjs@clsname
     {The current engine does not support the\MessageBreak
      'precise-text' option\@gobble}
  \else
    \bxjs@do@precisetext
  \fi
\fi
\g@addto@macro\bxjs@begin@document@hook{%
  \ifx\jsInhibitGlueAtParTop\@empty
    \def\bxjs@tmpa{\jsInhibitGlueAtParTop}%
    \ifx\everyparhook\bxjs@tmpa
      \let\everyparhook\@empty
    \fi
  \fi}
\ifnum\bxjs@everyparhook=\bxjs@everyparhook@@modern
  \let\bxjs@everypar\everypar
  \newtoks\everypar
  \everypar\bxjs@everypar
  \bxjs@everypar{\the\expandafter\everypar\everyparhook}%
\fi
\ifbxjs@fancyhdr
\@onlypreamble\bxjs@adjust@fancyhdr
\def\bxjs@adjust@fancyhdr{%
  \def\bxjs@tmpa{\fancyplain{}{\sl\rightmark}\strut}%
  \def\bxjs@tmpb{\fancyplain{}{\rightmark}\strut}%
  \ifx\f@ncyelh\bxjs@tmpa \global\let\f@ncyelh\bxjs@tmpb \fi
  \ifx\f@ncyerh\bxjs@tmpa \global\let\f@ncyerh\bxjs@tmpb \fi
  \ifx\f@ncyolh\bxjs@tmpa \global\let\f@ncyolh\bxjs@tmpb \fi
  \ifx\f@ncyorh\bxjs@tmpa \global\let\f@ncyorh\bxjs@tmpb \fi
  \def\bxjs@tmpa{\fancyplain{}{\sl\leftmark}\strut}%
  \def\bxjs@tmpb{\fancyplain{}{\leftmark}\strut}%
  \ifx\f@ncyelh\bxjs@tmpa \global\let\f@ncyelh\bxjs@tmpb \fi
  \ifx\f@ncyerh\bxjs@tmpa \global\let\f@ncyerh\bxjs@tmpb \fi
  \ifx\f@ncyolh\bxjs@tmpa \global\let\f@ncyolh\bxjs@tmpb \fi
  \ifx\f@ncyorh\bxjs@tmpa \global\let\f@ncyorh\bxjs@tmpb \fi
  \def\bxjs@tmpa{\rm\thepage\strut}%
  \def\bxjs@tmpb{\thepage\strut}%
  \ifx\f@ncyecf\bxjs@tmpa \global\let\f@ncyecf\bxjs@tmpb \fi
  \ifx\f@ncyocf\bxjs@tmpa \global\let\f@ncyocf\bxjs@tmpb \fi
  \ifx\fullwidth\@undefined\else \ifdim\textwidth<\fullwidth
    \setlength{\@tempdima}{\fullwidth-\textwidth}%
    \edef\bxjs@tmpa{\noexpand\fancyhfoffset[EL,OR]{\the\@tempdima}%
    }\bxjs@tmpa
  \fi\fi
  \PackageInfo\bxjs@clsname
   {Patch to fancyhdr is applied\@gobble}}
\def\bxjs@pagestyle@hook{%
  \@ifpackageloaded{fancyhdr}{%
    \bxjs@adjust@fancyhdr
    \global\let\bxjs@adjust@fancyhdr\relax
  }{}}
\let\bxjs@org@pagestyle\pagestyle
\def\pagestyle{%
  \bxjs@pagestyle@hook \bxjs@org@pagestyle}
\g@addto@macro\bxjs@begin@document@hook{%
  \bxjs@pagestyle@hook
  \global\let\bxjs@pagestyle@hook\relax}
\fi
\endinput
%%
%% End of file `bxjsja-minimal.def'.
